#include "tic_tac_toe.h"
#include "rectangle.h"

#include <SDL.h>
#include <iostream>

TicTacToe::TicTacToe(int width, int height, std::string title) {
    auto throw_error = [](std::string which) {
        throw std::runtime_error(which + " " + SDL_GetError());
    };

    // Alle Komponenten von SDL initialisieren
    // siehe https://wiki.libsdl.org/SDL_Init
    if (SDL_Init(SDL_INIT_EVERYTHING) < 0) {
        throw_error("SDL_Init fehlgeschlagen");
    }

    // Beim beenden SDL_Quit aufrufen (Resourcen freigeben)
    atexit(SDL_Quit);

    // Fenster erstellen, siehe https://wiki.libsdl.org/SDL_CreateWindow
    window.reset(SDL_CreateWindow(
        title.c_str(),
        SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
        width, height,
        SDL_WINDOW_RESIZABLE));

    if (!window) {
        throw_error("SDL_CreateWindow fehlgeschlagen");
    }

    // Renderer erstellen, siehe https://wiki.libsdl.org/SDL_CreateRenderer
    renderer.reset(SDL_CreateRenderer(window.get(), -1, 0));

    if (!renderer) {
        throw_error("SDL_CreateRenderer fehlgeschlagen");
    }

    // Masse speichern
    this->width = width;
    this->height = height;

    // Automatisch skalieren wenn sich Fenstergroesse aendert
	SDL_RenderSetLogicalSize(renderer.get(), width, height);

    // Fenster schwarz ausmalen
	SDL_RenderClear(renderer.get());

    // Aenderung sichtbar machen
	SDL_RenderPresent(renderer.get());

    // Eigener Initialisierungscode ab hier
    rect.reset(new Rectangle(Point(100, 100), 200, 200));
    rect->position = Point(100, 100);
    rect->setColor(255, 0, 0);
    rect->velocity = Point(5, 3);
}

bool TicTacToe::processEvents() {
    // Nutzereingaben verarbeiten
    SDL_Event evt;

    // Events einlesen
    // siehe https://wiki.libsdl.org/SDL_Event
    while (SDL_PollEvent(&evt)) {
        // Jeweils ein Event verarbeiten
        if (evt.type == SDL_QUIT) {
            // Benutzer hat Programmende angefordert
            return false;
        }
    }

    return true;
}

void TicTacToe::update() {
    rect->update();

    Point wsize = getWindowSize();

    // Bewege Rechteck wie im DVD-Bildschirmschoner
    if (rect->velocity.x > 0) {
        if (rect->position.x + rect->width > wsize.x) {
            rect->velocity.x = -rect->velocity.x;
        }
    } else {
        if (rect->position.x < 0) {
            rect->velocity.x = -rect->velocity.x;
        }
    }

    if (rect->velocity.y > 0) {
        if (rect->position.y + rect->height > wsize.y) {
            rect->velocity.y = -rect->velocity.y;
        }
    } else {
        if (rect->position.y < 0) {
            rect->velocity.y = -rect->velocity.y;
        }
    }
}

void TicTacToe::render() {
    SDL_SetRenderDrawColor(renderer.get(), 0xFF, 0xFF, 0xFF, 0xFF);
    SDL_RenderClear(renderer.get());

    rect->draw(renderer.get());

    SDL_RenderPresent(renderer.get());
}

Point TicTacToe::getWindowSize() {
    return Point(width, height);
}

